/*=====================================================================
  vars.h -> Variables used on the XSnake game.

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

 Please read the README file...
 (c) 1999 Santiago Romero Iglesias & Antonio Fernandez Fandos
=====================================================================*/


#define VELOCIDAD    70000            /* game delay */
#define INCREMENTO       3            /* snakes growing in blocks */
#define MAX_COMIDA       5            /* max apples on screen */
#define xmin             0            /* board dimensions */
#define xmax            79
#define ymin             0
#define ymax            39

#define true             1            /* some definitions */
#define false            0
#define boolean       char

#define VACIO            0            /* empty */
#define SERPIENTE        1            /* snake */
#define PARED            2            /* wall  */
#define COMIDA1          3            /* the 4 blocks of the apple */
#define COMIDA2          4
#define COMIDA3          5
#define COMIDA4          6
#define PODRIDA1         7            /* the 4 blocks of the lemon */
#define PODRIDA2         8
#define PODRIDA3         9
#define PODRIDA4        10
#define CABEZA_IZQ       6            /* more sprites (heads) */
#define CABEZA_DER       7
#define CABEZA_ARR       8
#define CABEZA_ABJO      9

#define DIR_IZQ          1            /* left direction */
#define DIR_DER          2            /* right */
#define DIR_ARR          3            /* up */
#define DIR_ABJO         4            /* down */

#define HAMBRE          25            /* # of apples to reach next level */

/* Key scancode, readed from /usr/X11R6/lib/X11/xkb/keycodes/xfree86 */
#define ESC_KEY          9            /* ESCAPE */
#define ARRIBA          98            /* cursor UP */
#define ABAJO          104            /* cursor DOWN */
#define IZQUIERDA      100            /* cursor LEFT */
#define DERECHA        102            /* cursor RIGHT */
#define ESPACIO         60            /* space */
#define ENTER         '\r'            /* enter */

#define CASTELLANO       0            /* language defines */
#define INGLES           1
#define MAX_IDIOMAS      2

/* probability of getting an APPLE instead of lemon, over 100 */
#define PROBAB_MANZANA  90

#define NUM_RECORDS      5


/*--- Global variables (sorry!!!) -----------------------*/
char done = 0,                    /* for the MAIN loop */
     tecla = 0,                   /* scancode of the key pressed */
     nivel = 0,                   /* game level */
     empezado=0;                  /* says when we've started
                                     the game to some functions */
char comida,                      /* # of apples on screen */
     idioma = 0,                  /* language */
     pausa = 0;                   /* pause status */
char muerto = 0,                  /* tells main loop if snake's dead */
     iniciado=0;
char hay_amarilla = 0,            /* indicates if there is any
                                     lemon on the screen */
     hay_record = 0;              /* indicates if there is an hi-score */
int tecla_ascii = 0;              /* ascii keypressed for the hi-score func*/

char pantalla[xmax+1][ymax+1],    /* array of screen characters, where
                                     we put 0 (empty), 1 (snake),
                                     2 (wall), 3 (apple), etc... */
     cola[xmax+1][ymax+1];        /* tail array, here we put the
                                     direction of the tail of each
                                     snake block, in order to be
                                     able to erase the tail each time */

char anchura_bloque = 7;
char altura_bloque  = 9;          /* block width and height */

char nombre[10];                  /* name for the hi-score function */
char offs=0;                      /* offset for that string */

Pixmap sprites[10];
unsigned long amarillo, verde,    /* colours and time */
              negro, blanco,
              azul, tiempo=0, segundos=0;

/* the snake structure */
struct serpiente
{
  int cabserx, cabsery,           /* snake's head (x,y) */
      colaserx, colasery,         /* snake's tail (x,y) */
      espera;                     /* when you eat, you put
                                     this to 5 to enlarge the snake */
  char izquierda, derecha, arriba, abajo;
  /* direction the snake is pointing to */

  int puntos, comidas;            /* points and apples eatten */
};

/* structs for time account */
struct timeval start, end;
long time_dif;

/* hi-score structures */
struct record
{
  int puntos;             /* score */
  int nivel;              /* level */
  char nombre[10];        /* name */
};

struct record records[NUM_RECORDS+1];

/*--- Function declarations -----------------------------*/
void InicializaMatrices( void );
void InicializaSerpiente( struct serpiente * );
void DibujaTablero( Display *, Pixmap, Drawable, GC );
void DibujaNivel( Display *, Pixmap, Pixmap, GC );
void LeerMovimiento( char, struct serpiente * );
void Movimiento( Display *, Drawable, Drawable, GC,
                 struct serpiente * );
void DibujaSerpiente( Display *, Pixmap, Window, GC,
                       struct serpiente * );
void DibujaSerpienteInicial( Display *, Pixmap, Window, GC,
                       struct serpiente * );
void EliminaCola( Display *, Pixmap, Window, GC,
                       struct serpiente * );
void BorraSerpiente( Display *, Pixmap, Window, GC,
                       struct serpiente * );
void ActualizaPuntos( Display *, Pixmap, Window, GC,
                       struct serpiente * );
void ModoPausa( Display *, Pixmap, Window, GC,
                       struct serpiente * );
void DibujaRecord( Display *, Pixmap, Window, GC,
                       struct serpiente * );
void HazRecord( Display *, Drawable, Drawable, GC,
                       struct serpiente * );
void DibujaComida( Display *, Drawable, Drawable, GC );
void BorraComida( Display *, Drawable, Drawable, GC, int, int );
void GeneraComida( Display *, Drawable, Drawable, GC );
void Presentacion( Display *, Drawable, Drawable, GC );
void DejarCola( struct serpiente * );
unsigned long ColorPorNombre( Display *, char *);
void RedibujaPantalla( Display *, Drawable, GC, Pixmap,
                       int, int, int, int  );
void DibujaBloque( Display *, Drawable, Drawable, GC, int, int, char );
void EscribeRecords( void );
void LeeRecords( void );
void OrdenaRecords( void );


/*--- Game messages, on Spanish and English--------------*/

char ErrorPixmap[MAX_IDIOMAS][60] =
 {
   "\nError creando pixmap.\nJuego abortado.\n\n",
   "\nError creating pixmap.\nGame aborted.\n\n"
 };

char HasConseguido[MAX_IDIOMAS][20] =
 {
   "Has conseguido ",
   "You scored "
 };

char Puntos1[MAX_IDIOMAS][10] =
 {
   "punto(s)",
   "point(s)"
 };

char Sobrevivido[MAX_IDIOMAS][25] =
 {
   "Has jugado durante",
   "You've played for"
 };

char Minutos[MAX_IDIOMAS][20] =
 {
   "minuto(s) y",
   "minute(s) and"
 };

char Segundos[MAX_IDIOMAS][15] =
 {
   "segundo(s)",
   "second(s)"
 };

char Puntos2[MAX_IDIOMAS][10] =
 {
   "PUNTOS: ",
   "SCORE: "
 };

char NombreA[MAX_IDIOMAS][10] =
 {
   "NOMBRE",
   "NAME "
 };

char NivelA[MAX_IDIOMAS][10] =
 {
   "NIVEL",
   "LEVEL"
 };


char PuntosA[MAX_IDIOMAS][10] =
 {
   "PUNTOS",
   "SCORE"
 };

char Nivel[MAX_IDIOMAS][10] =
 {
   "NIVEL: ",
   "LEVEL: "
 };

char Pausa[MAX_IDIOMAS][10] =
 {
   "EN PAUSA",
   " PAUSED "
 };

char Records[MAX_IDIOMAS][25] =
 {
   "PUNTUACIONES MAXIMAS",
   "     HI SCORES      "
 };

char Records2[MAX_IDIOMAS][30] =
 {
   "ENHORABUENA, GRAN PUNTUACION",
   "CONGRATULATIONS, GREAT SCORE!"
 };

char Records3[MAX_IDIOMAS][55] =
 {
   "Escriba su nombre y pulse ENTER para terminar.",
   "Write your name and press ENTER to finish."
 };


char PulseF1[MAX_IDIOMAS][50] =
 {
   "Pulse ESPACIO para comenzar una nueva partida.",
   "Press the SPACE key to start a new game."
 };

char Instrucciones[MAX_IDIOMAS][70] =
 {
   "Recoja 25 manzanas por nivel usando los cursores.",
   "Collect 25 apples on each level using cursor keys."
 };

char Ayuda[MAX_IDIOMAS][1000] =
 {
   " XSnake es la version Linux del sencillo juego Snake, donde\n\
 una serpiente debe recoger las manzanas que aparecen por la\n\
 pantalla, subiendo de nivel. Cada manzana alarga la cola de la\n\
 serpiente un poco ms, haciendo ms dificil el juego en cada\n\
 nivel.\n\n\
 Uso del programa:    xsnake [opciones]\n\n\
   OPCIONES:\n\
       --ayuda           Esta ayuda.\n\
       --grande          Tamao de pantalla grande.\n\
       --extragrande     Tamao de pantalla extragrande.\n\n\
   TECLAS:\n\
       Cursores          Cambiar la direccin de la serpiente.\n\
       ESCAPE            Salir del juego.\n\
       Espacio           Pausa / reanudar juego.\n",

   " XSnake is the Linux version of the clasic Snake game, where\n\
 a snake must collect all the apples that appear on the screen.\n\
 Each apple makes the snake to grow, making the game more and more\n\
 difficult each level.\n\n\
 Program usage:       xsnake [options]\n\n\
   OPTIONS:\n\
       --help            This help.\n\
       --big             Big screen game.\n\
       --extrabig        Extra big screen game.\n\n\
   KEYS:\n\
       Cursors           Change snake's direction.\n\
       ESCAPE            Exit game.\n\
       Space            Pause / resume game.\n"
 };
