/*=====================================================================
  Xsnake.c -> XWindow version of the SNAKE game (a classic of
              the first arcade games). On this game you move
              a snake over the screen, collecting apples, which
              makes the snake larger. After collecting 25 apples,
              you go to a new level where you get MORE larger
              for each apple you get. The goal is just to reach
              the further level you can, collecting the more
              apples possible, and so, trying to do the best
              score in the hi-score chart.

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

 Please read the README file...
 (c) 1999 Santiago Romero Iglesias & Antonio Fernandez Fandos
=====================================================================*/

/*--- Header file used by the program -------------------------------*/
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <stdio.h>
#include <X11/extensions/shape.h>
#include <Imlib.h>
#include <sys/time.h>
#include <unistd.h>

#include "vars.h"

/*=== main function main() ==========================================*/
int main( char argc, int *argv[])
{

  Display *display;                      /* connection with XServer */
  Window window,                         /* game main window        */
         rootwin;                        /* root window (desktop)   */
  Pixmap pixmap;                         /* the virtual screen      */
  GC gc;
  ImlibData *id;                         /* Imlib stuff             */
  ImlibImage *imagen;
  XSetWindowAttributes attr;
  XEvent evento;
  Font fuente;                           /* attribs, events & fonts */
  XSizeHints myhints;

  int pant, anchura, altura;
  char key;
  char *lang;
  struct serpiente serpiente1;           /* the player's snake      */

  printf("XSnake - v2.0  (c) 1999 Santiago Romero ");
  printf("& Antonio Fernandez.\n\n");

  /* first of all we read the LANG variable of the O.S. to
     know which language prefers to use the player, between
     Spanish and English                                            */


  lang = getenv("LANG");
  if( lang != NULL )
  {
    if( strstr(lang, "es") != NULL || strstr(lang, "ES") != NULL )
       idioma = CASTELLANO;
    else
       idioma = INGLES;
  }
  else
    idioma = INGLES;


  /* now we're going to read the command line to look for
     the game switches (--help, --big, etc.)                        */
  if( argc > 1 )
  {
     if( strstr( (const char *) &(argv[1][0]), "--help") != NULL ||
         strstr( (const char *) &(argv[1][0]), "--ayuda") != NULL ||
         strstr( (const char *) &(argv[1][0]), "-?") != NULL ||
         strstr( (const char *) &(argv[1][0]), "-help") != NULL ||
         strstr( (const char *) &(argv[1][0]), "-ayuda") != NULL ||
         strstr( (const char *) &(argv[1][0]), "-h") != NULL ||
         strstr( (const char *) &(argv[1][0]), "/?") != NULL )
     {
        printf("%s\n", Ayuda[idioma]);
        exit(0);
     }
     if( strstr( (const char *) &(argv[1][0]), "--big") != NULL ||
       strstr( (const char *) &(argv[1][0]), "--grande") != NULL)
     {
        anchura_bloque = 10;
        altura_bloque = 12;
     }

     if( strstr( (const char *) &(argv[1][0]), "--extrabig") != NULL ||
       strstr( (const char *) &(argv[1][0]), "--extragrande") != NULL)
     {
        anchura_bloque = 12;
        altura_bloque = 14;
     }
  }

  /* read the game hi-score from HOME/.xsnake.hi */
  LeeRecords();

  /* now we open the main window centered on the screen
     and create the pixmaps/screens/ImlibImages needed */
  display = XOpenDisplay( NULL );
  if( display == (Display *) NULL )
  {
    printf("Couldn't open DISPLAY...\n");
    printf("(This is an X-Window program).\n");
    exit(1);
  }

  XGetGeometry( display, XDefaultRootWindow(display), &rootwin,
                &pant, &pant, &anchura, &altura, &pant, &pant );

  id = Imlib_init(display);
  window = XCreateWindow(display,
             DefaultRootWindow(display),
             (anchura/2)-((anchura_bloque*(xmax+1))/2),
             (altura/2)-((altura_bloque*(ymax+1)+40)/2),
             anchura_bloque*(xmax+1), altura_bloque*(ymax+1)+40, 0,
             id->x.depth, InputOutput, id->x.visual, 0, &attr);

  myhints.min_width  =
  myhints.max_width  = anchura_bloque*(xmax+1) ;
  myhints.min_height =
  myhints.max_height = altura_bloque*(ymax+1)+40 ;
  myhints.x = (anchura/2)-((anchura_bloque*(xmax+1))/2);
  myhints.y = (altura/2)-((altura_bloque*(ymax+1)+40)/2);
  myhints.flags = PMinSize | PMaxSize | PPosition;
  XSetNormalHints( display, window, &myhints );

  pant = DefaultScreen(display);
  XSelectInput( display, window, KeyPressMask | ExposureMask );
  XStoreName(display, window, "X-Snake");
  pixmap = XCreatePixmap( display, window,
              anchura_bloque*(xmax+1), altura_bloque*(ymax+1)+40,
              DefaultDepth(display, pant));

  if( pixmap == (Pixmap) NULL )
  {
    printf("%s", ErrorPixmap[idioma]);
    XCloseDisplay(display);
    exit(1);
  }

  /* get some standard colors by it's name */
  negro = ColorPorNombre( display, "black" );
  amarillo = ColorPorNombre(display, "yellow" );
  verde = ColorPorNombre( display, "green" );
  azul = ColorPorNombre( display, "lightblue" );
  blanco = ColorPorNombre( display, "white" );

  gc = XCreateGC( display, pixmap, 0, NULL );
  XSetForeground(display, gc, negro );
  XFillRectangle( display, pixmap, gc, 0, 0,
       anchura_bloque*(xmax+1), altura_bloque*(ymax+1)+40 );

  /* load the standard 9x15 font */
  fuente = XLoadFont( display, "9x15" );
  XSetFont( display, gc, fuente );

  /* load the JPG game images and convert them into pixmaps */
  imagen=Imlib_load_image(id, "/usr/games/xsnake/xsnake0.jpg");
  Imlib_render( id, imagen, anchura_bloque, altura_bloque );
  sprites[0] = (Pixmap) Imlib_move_image( id, imagen );
  imagen=Imlib_load_image(id, "/usr/games/xsnake/xsnake1.jpg");
  Imlib_render( id, imagen, anchura_bloque, altura_bloque );
  sprites[1] = (Pixmap) Imlib_move_image( id, imagen );
  imagen=Imlib_load_image(id, "/usr/games/xsnake/xsnake2.jpg");
  Imlib_render( id, imagen, anchura_bloque, altura_bloque );
  sprites[2] = (Pixmap) Imlib_move_image( id, imagen );
  imagen=Imlib_load_image(id, "/usr/games/xsnake/xsnake3.jpg");
  Imlib_render( id, imagen, anchura_bloque*2, altura_bloque*2 );
  sprites[3] = (Pixmap) Imlib_move_image( id, imagen );
  imagen=Imlib_load_image(id, "/usr/games/xsnake/xsnake.jpg");
  Imlib_render( id, imagen, 238, 128 );
  sprites[4] = (Pixmap) Imlib_move_image( id, imagen );
  imagen=Imlib_load_image(id, "/usr/games/xsnake/xsnake4.jpg");
  Imlib_render( id, imagen, anchura_bloque*2, altura_bloque*2 );
  sprites[5] = (Pixmap) Imlib_move_image( id, imagen );
  imagen=Imlib_load_image(id, "/usr/games/xsnake/xsnake5.jpg");
  Imlib_render( id, imagen, anchura_bloque, altura_bloque );
  sprites[6] = (Pixmap) Imlib_move_image( id, imagen );
  imagen=Imlib_load_image(id, "/usr/games/xsnake/xsnake6.jpg");
  Imlib_render( id, imagen, anchura_bloque, altura_bloque );
  sprites[7] = (Pixmap) Imlib_move_image( id, imagen );
  imagen=Imlib_load_image(id, "/usr/games/xsnake/xsnake7.jpg");
  Imlib_render( id, imagen, anchura_bloque, altura_bloque );
  sprites[8] = (Pixmap) Imlib_move_image( id, imagen );
  imagen=Imlib_load_image(id, "/usr/games/xsnake/xsnake8.jpg");
  Imlib_render( id, imagen, anchura_bloque, altura_bloque );
  sprites[9] = (Pixmap) Imlib_move_image( id, imagen );

  /* init some variables */
  serpiente1.puntos = 0;
  nivel = tiempo = segundos = 0;
  srand( time(NULL) );
  records[NUM_RECORDS].puntos = 0;
  hay_record = 0;

  /* display the window */
  XMapWindow(display, window);

  /* first of all, we clear all the events until the
     window is finally up */
  while (1)
  {
     XNextEvent(display, &evento);
     if (evento.xany.window != window) continue;
     if (evento.type == Expose) break;
  }


  /*--- The main loop starts here -----------------------------------*/
  while ( !done )
  {

   /* first we get the time in useconds in order to
      do the right delay after the main loop */
   gettimeofday(&start, NULL);

   /* next follows the event fetch. Each loop we take
      care of all possible events of our application
      and then we enter on the code where we create
      and display the next game frame */

   while (QLength(display) > 0)
   {

    /* get event */
    XNextEvent(display, &evento);
    if (evento.xany.window != window) continue;

    /* look which event we got */
    switch(evento.type)
    {
     /* expose: we must redraw the screen from the
        pixmap we're drawing to (as a virtual screen) */
     case Expose:
           RedibujaPantalla(display, window, gc, pixmap,
                 evento.xexpose.x, evento.xexpose.y,
                 evento.xexpose.width, evento.xexpose.height );
         XFlush(display);
         break;

     /* keypress: we get the key pressed (both ascii code and
        scancode) and leave them on global variables (ugh) so
        that the rest of the program can know what to do */

     case KeyPress:
         key = XKeycodeToKeysym(display, evento.xkey.keycode, 0);
         tecla_ascii = key;

         /* the hay_record variable is true when we are writing
            the hi_scores, where SPACE is not available :) */
         if( key == ' ' && !hay_record )
         {
            if( iniciado == 2 )
               ModoPausa( display, pixmap, window, gc, &serpiente1 );
            else
               key = ENTER;
         }

         /* the enter key, when the game has not started
            (iniciado != 2 ) and we're not writing a
            hiscore (!hay_record) makes the game start. */
         if( key == ENTER && iniciado != 2  && !hay_record )
         {
            /* start game = reset variables, redraw screen, etc. */
            pausa = 0;
            serpiente1.puntos = 0;
            muerto = 1;
            iniciado = 2;
            XSetForeground(display, gc, negro );
            XFillRectangle( display, pixmap, gc, anchura_bloque,
              altura_bloque, anchura_bloque*(xmax-1), altura_bloque*(ymax-1));
            RedibujaPantalla(display, window, gc, pixmap,
            0, 0, anchura_bloque*(xmax+1), altura_bloque*(ymax+1)+40 );
            key = 0;
            evento.xkey.keycode = 0;
            nivel = 0;
            hay_amarilla = 0;
            DibujaNivel( display, pixmap, window, gc );
         }

         /* ESC key -> exit main loop! */
         else if( evento.xkey.keycode == ESC_KEY )
            done = 1;

         /* other keys, save them to the tecla variable */
         else
         {
            if( !pausa && iniciado == 2  )
               tecla = evento.xkey.keycode;
         };
         break;

    default: break;
    }
 }

 /* here starts the code where we create each frame */
    /* if we're not still playing (!iniciado) and we're
       not on pause (!pausa), we test if there is an
       hi-score */

    if( !iniciado && !pausa )
    {
       if( serpiente1.puntos > records[NUM_RECORDS-1].puntos &&
           hay_record != 2 )
       {
          offs = 0;
          hay_record = 1;
          tecla_ascii = 0;
          pausa = 1;
        }
        records[NUM_RECORDS].nivel = nivel+1;
    }

    /* if we have detected an hi-score, go to the function HazRecord */
    if( hay_record  > 0 )
    {
      if( hay_record == 1 )
      {
        DibujaRecord( display, pixmap, window, gc, &serpiente1 );
        hay_record = 2;
      }
      HazRecord( display, pixmap, window, gc, &serpiente1 );
    }

   /* this is the main main loop (not paused, not hi-score) */
   else
    {
       /* this block stands inicialization on each death
          and new level, donde by setting iniciado=0 */
       if( !iniciado )
       {

          /* if dead (muerto) variable == 1 ... */
          if( muerto )
          {
            pausa = 0;
            hay_amarilla = 0;
            sleep(1);
          }

          tecla = comida = pausa = 0;

          srand( time(NULL) );
          InicializaMatrices();
          InicializaSerpiente( &serpiente1 );
          DibujaTablero( display, pixmap, window, gc );
          ActualizaPuntos(display, pixmap, window, gc, &serpiente1 );
          Presentacion( display, window, pixmap, gc );
          XMapWindow( display, window );
          iniciado = 1;
        }

        if( muerto )
        {

          offs = 0;
          if( iniciado == 2 )
          {
            DibujaSerpienteInicial(display, pixmap, window, gc, &serpiente1 );
            InicializaSerpiente( &serpiente1 );
            ActualizaPuntos(display, pixmap, window, gc, &serpiente1);
          }
          XMapWindow( display, window );
          empezado = muerto = 0;
        }

        /* The MAIN MAIN MAIN MAIN game loop: here we detect
           if the tecla variable (set by the event loop) is
           != 0 and then we move the snake, draw it and test
           if it went over an apple or wall */
        if( !pausa && iniciado == 2 )
        {

           if( tecla != 0 )
              LeerMovimiento ( tecla, &serpiente1 );

           GeneraComida( display, pixmap, window, gc );
           if( empezado )
           {
              Movimiento( display, pixmap, window, gc, &serpiente1 );
              DibujaSerpiente(display, pixmap, window, gc, &serpiente1 );
              BorraSerpiente(display, pixmap, window, gc, &serpiente1 );
           }
           else
           {
              DibujaSerpienteInicial(display, pixmap, window, gc, &serpiente1 );
           }
        }
   }

        /* now we take the end time */
        gettimeofday(&end, NULL);

        /* at this moment we calc the elapsed time
           between the start of the loop and this
           moment: */
        if( end.tv_usec < start.tv_usec )
            end.tv_usec += 1000000;
        time_dif = end.tv_usec - start.tv_usec;

        XSync(display, False);

        /* we must now make our game stop till the
           next frame must be written, so we're going
           to usleep() a number of microseconds
           depending on the speed we want for our snake.
           If we select 100.000 us, our snake will move
           10 times per second (10 game frames per sec)
           (as 1 sec = 1.000.000/100.000 = 10).

           We could do an usleep(VELOCIDAD); but we must
           count on the time elapsed fetching events,
           so now we substract that time_dif we've
           calculated before */

        usleep(VELOCIDAD-time_dif);

        /* implementamos un cronometro de useg a segundos */
        tiempo += VELOCIDAD;
        if( tiempo > 1000000 )
        {
           tiempo -= 1000000;
           segundos++;
        }
 }

  /* now we print the exit messages in the right language */
  printf(" %s", HasConseguido[idioma] );
  printf("%d", serpiente1.puntos );
  printf(" %s.\n", Puntos1[idioma] );
  printf(" %s", Sobrevivido[idioma] );
  printf(" %d ", segundos/60);
  printf("%s", Minutos[idioma] );
  printf(" %d ", segundos%60);
  printf("%s.\n\n", Segundos[idioma] );
  XUnmapWindow(display, window);
  XUnloadFont(display, fuente);
  XFreePixmap(display, pixmap);
  XFreeGC( display, gc );
  XDestroyWindow(display, window );
  XFlush( display );
  XCloseDisplay(display);
  exit( 0 );
}


/*------------------------------------------------------------
  Init the game matrixes setting all them to 0 (VACIO=EMPTY)
 ------------------------------------------------------------*/
void InicializaMatrices( void )
{
  int x, y;
  for( y=0; y<ymax+1; y++ )
    for( x=0; x<xmax+1; x++ )
       pantalla[x][y] = cola[x][y] = VACIO;
}


/*---------------------------------------------------------
  Inits the snake basic valores (x, y, etc...)
 --------------------------------------------------------*/
void InicializaSerpiente( struct serpiente *serp )
{
  serp->cabserx = xmax/2;
  serp->cabsery = ymax/2;
  serp->colaserx = xmax/2;
  serp->colasery = ymax/2;
  serp->espera = (INCREMENTO*2);
  serp->izquierda = false;
  serp->derecha = false;
  serp->arriba = false;
  serp->abajo = false;
  serp->comidas = 0;

  pantalla[serp->colaserx][serp->colasery] = SERPIENTE;
  cola[serp->colaserx][serp->colasery] = DIR_DER;
}


/*---------------------------------------------------------
  Draws the board on the window and on the Pixmap.
 --------------------------------------------------------*/
void DibujaTablero( Display *display, Pixmap pixmap,
                     Drawable window, GC gc )
{
  int x, y;
  char cad[4];

  XSetForeground(display, gc, negro);
  XFillRectangle( display, pixmap, gc, 0, 0,
    anchura_bloque*(xmax+1), altura_bloque*(ymax+1)+40 );

  /* draw blocks */
  for( x=0; x<=xmax; x++ )
  {
    DibujaBloque( display, pixmap, window, gc, x, ymin, PARED );
    DibujaBloque( display, pixmap, window, gc, x, ymax, PARED );
  }

  for( y=0; y<=ymax; y++ )
  {
    DibujaBloque( display, pixmap, window, gc, xmin, y, PARED );
    DibujaBloque( display, pixmap, window, gc, xmax, y, PARED );
  }

  /* draw texts (SCORE, LEVEL, X-SNAKE) */
  XSetForeground(display, gc, verde);
  XDrawString( display, pixmap, gc, 10,
               ((altura_bloque*ymax)+40)-5,
                Puntos2[idioma], strlen(Puntos2[idioma]) );

  XDrawString( display, pixmap, gc, 105+70,
               ((altura_bloque*ymax)+40)-5,
                Nivel[idioma], strlen(Nivel[idioma]) );

  XDrawString( display, pixmap, gc, (anchura_bloque*(xmax-1))-62,
               ((altura_bloque*ymax)+40)-5, "X-SNAKE", 7 );

  /* draw food rectangle */
  XSetForeground(display, gc, amarillo);
  XDrawRectangle( display, pixmap, gc, 105+55+120,
                  ((altura_bloque*ymax)+40)-20, (HAMBRE*4)+4, 20 );

  sprintf(cad, "%d", nivel+1 );
  XDrawString( display, pixmap, gc, 105+55+81,
               ((altura_bloque*ymax)+40)-5, cad, strlen(cad) );

  /* all was written on PIXMAP, now flip it to WINDOW */
  RedibujaPantalla(display, window, gc, pixmap,
   0, 0, anchura_bloque*(xmax+1), altura_bloque*(ymax+1)+40 );
}

/*---------------------------------------------------------
  Draws the level on the screen.
 --------------------------------------------------------*/
void DibujaNivel( Display *display, Pixmap pixmap,
                  Pixmap window, GC gc )
{

  char cad[5];

  XSetForeground(display, gc, negro);
  XFillRectangle( display, pixmap, gc, 105+55+80,
                  ((altura_bloque*ymax)+40)-20, 30, 20 );
  XFillRectangle( display, window, gc, 105+55+80,
                  ((altura_bloque*ymax)+40)-20, 30, 20 );

  XSetForeground(display, gc, amarillo);
  sprintf(cad, "%d", nivel+1 );
  XDrawString( display, pixmap, gc, 105+55+81,
               ((altura_bloque*ymax)+40)-5, cad, strlen(cad) );
  XDrawString( display, window, gc, 105+55+81,
               ((altura_bloque*ymax)+40)-5, cad, strlen(cad) );

}

/*---------------------------------------------------------
  Writes the score on the screen, by deleting the
  old one with a black rectangle :)
 --------------------------------------------------------*/
void ActualizaPuntos( Display *display, Pixmap pixmap,
         Drawable window, GC gc, struct serpiente *serp )
{
  char puntos[7];

  sprintf(puntos, "%d", serp->puntos);

  /* first delete the old score */
  XSetForeground(display, gc, negro);
  XFillRectangle( display, pixmap, gc, 75,
                  ((altura_bloque*ymax)+40)-20, 6*9, 20 );
  XFillRectangle( display, window, gc, 75,
                  ((altura_bloque*ymax)+40)-20, 6*9, 20 );

  /* now draw the new one */
  XSetForeground(display, gc, amarillo);
  XDrawString( display, window, gc, 80,
                  ((altura_bloque*ymax)+40)-5, puntos, strlen(puntos) );
  XDrawString( display, pixmap, gc, 80,
                  ((altura_bloque*ymax)+40)-5, puntos, strlen(puntos) );

  /* draw too the APPLE-BAR :) if has eaten > 0 apples */
  if( serp->comidas > 0 )
  {
     XFillRectangle( display, window, gc, 105+55+122+(serp->comidas*4)-4,
                  ((altura_bloque*ymax)+40)-17, 4, 15 );
     XFillRectangle( display, pixmap, gc, 105+55+122+(serp->comidas*4)-4,
                  ((altura_bloque*ymax)+40)-17, 4, 15 );
  }
}


/*---------------------------------------------------------
  Acts on the pause toggling by drawing the "PAUSED"
  message or redrawing the APPLE-BAR :)
 --------------------------------------------------------*/
void ModoPausa( Display *display, Pixmap pixmap,
        Drawable window, GC gc, struct serpiente *serp )
{
  /* toggle the pause status */
  pausa = !pausa;

  if( pausa  )
  {
      XSetForeground(display, gc, negro );
      XFillRectangle( display, window, gc, 105+55+122,
                  ((altura_bloque*ymax)+40)-17, (serp->comidas*4), 15 );
      XFillRectangle( display, pixmap, gc, 105+55+122,
                  ((altura_bloque*ymax)+40)-17, (serp->comidas*4), 15 );
      XSetForeground(display, gc, verde );
      XDrawString( display, window, gc, 105+55+135,
                  ((altura_bloque*ymax)+40)-5,
                  Pausa[idioma], strlen(Pausa[idioma]) );
      XDrawString( display, pixmap, gc, 105+55+135,
                  ((altura_bloque*ymax)+40)-5,
                  Pausa[idioma], strlen(Pausa[idioma]) );
  }
  else
  {
      XSetForeground(display, gc, negro );
      XFillRectangle( display, window, gc, 105+55+122,
                  ((altura_bloque*ymax)+40)-17, HAMBRE*4, 15 );
      XFillRectangle( display, pixmap, gc, 105+55+122,
                  ((altura_bloque*ymax)+40)-17, HAMBRE*4, 15 );
      XSetForeground(display, gc, amarillo );
      XFillRectangle( display, window, gc, 105+55+122,
                  ((altura_bloque*ymax)+40)-17, (serp->comidas*4), 15 );
      XFillRectangle( display, pixmap, gc, 105+55+122,
                  ((altura_bloque*ymax)+40)-17, (serp->comidas*4), 15 );
  }
}


/*---------------------------------------------------------
  Changes snake.direction depending on the keycode.
 --------------------------------------------------------*/
void LeerMovimiento( char keycode, struct serpiente *serp )
{

   if( keycode == ARRIBA && serp->abajo == false )
   {
      serp->arriba = true;
      serp->derecha = serp->izquierda = serp->abajo = false;
   }

   else if( keycode == ABAJO && serp->arriba == false )
   {
      serp->abajo = true;
      serp->derecha = serp->arriba = serp->izquierda = false;
   }

   else if( keycode == IZQUIERDA && serp->derecha == false)
   {
      serp->izquierda = true;
      serp->derecha = serp->arriba = serp->abajo = false;
   }

   else if( keycode == DERECHA && serp->izquierda == false)
   {
      serp->derecha = true;
      serp->arriba = serp->izquierda =  serp->abajo = false;
   }

   /* the first movement we do (when the snake is stopped
      at the center of the screen) needs to write the
      tail direction on the array */
   if( !empezado )
   {
      if( tecla == ARRIBA )
      {
        cola[serp->colaserx][serp->colasery] = DIR_ARR;
        empezado = 1;
      }
      else if( tecla == ABAJO )
      {
        cola[serp->colaserx][serp->colasery] = DIR_ABJO;
        empezado = 1;
      }
      else if( tecla == IZQUIERDA )
      {
        cola[serp->colaserx][serp->colasery] = DIR_IZQ;
        empezado = 1;
      }
      else if( tecla == DERECHA )
      {
        cola[serp->colaserx][serp->colasery] = DIR_DER;
        empezado = 1;
      }
      /* with empezado = 1 we are saying to the main
         loop that we're now PLAYING */
   }

   /* clear the key */
   tecla = 0;
}



/*---------------------------------------------------------
  Moves the snake depending off snake->direction.
 --------------------------------------------------------*/
void Movimiento( Display *display, Pixmap pixmap,
                 Window window, GC gc, struct serpiente *serp )
{
   int dircola, cabserantx, cabseranty;
   char dato;

   /* save those variables */
   cabserantx = serp->cabserx;
   cabseranty = serp->cabsery;

   /* test directions, change the snake's X,Y */
   if ( serp->izquierda == true )
   {
     /* test if we're death (walls) */
     if ( serp->cabserx > xmin+1 )
     {
        serp->cabserx = (serp->cabserx)-1;
        dircola = DIR_IZQ;
     }
     else { muerto = 1; iniciado = 0; }
   }

   else if ( serp->derecha == true )
   {
     if ( serp->cabserx < xmax-1 )
     {
        serp->cabserx = (serp->cabserx)+1;
        dircola = DIR_DER;
     }
     else { muerto = 1; iniciado = 0; }
   }

   else if ( serp->arriba == true )
   {
     if ( serp->cabsery > ymin+1 )
     {
        serp->cabsery = (serp->cabsery)-1;
        dircola = DIR_ARR;
     }
     else { muerto = 1; iniciado = 0; }
   }

   else if ( serp->abajo == true )
   {
     if ( serp->cabsery < ymax-1 )
     {
        serp->cabsery = (serp->cabsery)+1;
        dircola = DIR_ABJO;
     }
     else { muerto = 1; iniciado = 0; }
   }

   /* test if we've crashed with ourselves ...
      if so, make muerto=1 (dead=1) to ack
      it to the main loop :) */
   if( pantalla[serp->cabserx][serp->cabsery] == SERPIENTE ||
       pantalla[serp->cabserx][serp->cabsery] == SERPIENTE )
     { muerto = 1; iniciado = 0; }

   /* now test for APPLES */
   else
   {
     /* test for red apples */
     if( pantalla[serp->cabserx][serp->cabsery] >= COMIDA1 )
     {
        /* test if is a LEMON */
        if( pantalla[serp->cabserx][serp->cabsery] >= PODRIDA1 )
        {
           hay_amarilla = 0;
           DejarCola( serp );
           serp->puntos = serp->puntos += 10;
        }
        /* then is a red apple */
        else
        {
          serp->espera = serp->espera + (INCREMENTO*(nivel+1));
          serp->puntos = serp->puntos ++;
        }

        /* clear apple/lemon and EAT :) */
        BorraComida( display, pixmap, window, gc,
                      serp->cabserx, serp->cabsery );
        comida--;
        serp->comidas++;
        ActualizaPuntos(display, pixmap, window, gc, serp );

        /* test if we've reached the 25 apples goal */
        if( serp->comidas >= HAMBRE )
        {
           nivel++;
           hay_amarilla = 0;
           DibujaTablero( display, pixmap, window, gc );

           InicializaMatrices();
           serp->puntos += 100;
           comida = tecla = empezado = 0;
           ActualizaPuntos(display, pixmap, window, gc, serp );
           InicializaSerpiente( serp );
           DibujaSerpienteInicial(display, pixmap, window, gc, serp );

           XSetForeground(display, gc, negro);
           XFillRectangle( display, window, gc, 105+55+122,
                  ((altura_bloque*ymax)+40)-17, (HAMBRE*4), 15 );
           XFillRectangle( display, pixmap, gc, 105+55+122,
                  ((altura_bloque*ymax)+40)-17, (HAMBRE*4), 15 );
           XSetForeground(display, gc, amarillo);
           return;
        }

     }

      pantalla[serp->cabserx][serp->cabsery] = SERPIENTE;
      cola[cabserantx][cabseranty] = dircola;
   }
}


/*---------------------------------------------------------
  Leave the snake tail on the screen by modifying the
  colaserx and colasery variables.
 --------------------------------------------------------*/
void DejarCola( struct serpiente *serp )
{
   serp->colaserx = serp->cabserx;
   serp->colasery = serp->cabsery;
   serp->espera = (INCREMENTO*2);
}


/*---------------------------------------------------------
  Creates and draws the APPLE/LEMON graphic.
 --------------------------------------------------------*/
void DibujaComida( Display *display, Pixmap pixmap,
                   Window window, GC gc )
{
   int posx, posy, contador = 0, cual;

   /* we're going to try 100 times to find an
      slot on the screen empty for our fruit,
      as it occupies 4 times the snake/wall
      space... */
   do
   {
      posx = rand() % (xmax-3)+1;
      posy = rand() % (ymax-3)+1;
      contador++;
   }
   while( pantalla[posx][posy] != VACIO ||
          pantalla[posx+1][posy] != VACIO ||
          pantalla[posx][posy+1] != VACIO ||
          pantalla[posx+1][posy+1] != VACIO &&
          contador < 100 );

   if( contador < 100 )
   {
     /* now we're going to see if we put APPLE or LEMMON */
     if( ((rand() % 100)<PROBAB_MANZANA) || hay_amarilla )
     {
        pantalla[posx][posy] = COMIDA1;
        pantalla[posx+1][posy] = COMIDA2;
        pantalla[posx][posy+1] = COMIDA3;
        pantalla[posx+1][posy+1] = COMIDA4;
        cual = 3;                        /* sprite = apple */
     }

     else
     {
        pantalla[posx][posy] = PODRIDA1;
        pantalla[posx+1][posy] = PODRIDA2;
        pantalla[posx][posy+1] = PODRIDA3;
        pantalla[posx+1][posy+1] = PODRIDA4;
        hay_amarilla = 1;
        cual = 5;                        /* sprite = lemon */
     }

     /* draw the right sprite */
     XCopyArea(display, sprites[cual], pixmap, gc,
           0, 0, anchura_bloque*2, altura_bloque*2, posx*anchura_bloque,
                                              posy*altura_bloque);
     XCopyArea(display, sprites[cual], window, gc,
           0, 0, anchura_bloque*2, altura_bloque*2, posx*anchura_bloque,
                                              posy*altura_bloque);

      comida++;
  }
}


/*---------------------------------------------------------
  Erases the fruit graphic (by drawing a black sprite).
 --------------------------------------------------------*/
void BorraComida( Display *display, Pixmap pixmap,
                    Window window, GC gc, int x, int y )
{
  /* test on which of the 4 apple/lemon blocks we've
     impacted and erase all 4 blocks */
  switch( pantalla[x][y] )
  {
     case COMIDA1:
     case PODRIDA1:
       pantalla[x+1][y] = pantalla[x][y+1] = pantalla[x+1][y+1] = VACIO;
       DibujaBloque( display, pixmap, window, gc, x+1, y, VACIO );
       DibujaBloque( display, pixmap, window, gc, x, y+1, VACIO );
       DibujaBloque( display, pixmap, window, gc, x+1, y+1, VACIO );
       break;

     case COMIDA2:
     case PODRIDA2:
       pantalla[x-1][y] = pantalla[x][y+1] = pantalla[x-1][y+1] = VACIO;
       DibujaBloque( display, pixmap, window, gc, x-1, y, VACIO );
       DibujaBloque( display, pixmap, window, gc, x, y+1, VACIO );
       DibujaBloque( display, pixmap, window, gc, x-1, y+1, VACIO );
       break;

     case COMIDA3:
     case PODRIDA3:
       pantalla[x+1][y] = pantalla[x][y-1] = pantalla[x+1][y-1] = VACIO;
       DibujaBloque( display, pixmap, window, gc, x+1, y, VACIO );
       DibujaBloque( display, pixmap, window, gc, x, y-1, VACIO );
       DibujaBloque( display, pixmap, window, gc, x+1, y-1, VACIO );
       break;

     case COMIDA4:
     case PODRIDA4:
       pantalla[x][y-1] = pantalla[x-1][y] = pantalla[x-1][y-1] = VACIO;
       DibujaBloque( display, pixmap, window, gc, x, y-1, VACIO );
       DibujaBloque( display, pixmap, window, gc, x-1, y, VACIO );
       DibujaBloque( display, pixmap, window, gc, x-1, y-1, VACIO );
       break;
  }
   pantalla[x][y] = SERPIENTE;
}


/*---------------------------------------------------------
  Creates food if necesary.
 --------------------------------------------------------*/
void GeneraComida( Display *display, Pixmap pixmap,
                    Window window, GC gc )
{
   /* only 5 pieces of fruit on the screen each time */
   if( comida < MAX_COMIDA )
      DibujaComida( display, pixmap, window, gc );
}


/*---------------------------------------------------------
  Draws the snake graphic. It takes care of the directorio
  to draw the right snake head and erase the old one.
 --------------------------------------------------------*/
void DibujaSerpiente( Display *display, Pixmap pixmap,
                       Window window, GC gc,
                       struct serpiente *serp )
{
  if( !empezado ) return;

   if( serp->derecha )
   {
       DibujaBloque( display, pixmap, window, gc,
              serp->cabserx-1, serp->cabsery, SERPIENTE );
       DibujaBloque( display, pixmap, window, gc,
              serp->cabserx, serp->cabsery, CABEZA_DER );
   }
   else if( serp->arriba )
   {
       DibujaBloque( display, pixmap, window, gc,
              serp->cabserx, serp->cabsery+1, SERPIENTE );
       DibujaBloque( display, pixmap, window, gc,
              serp->cabserx, serp->cabsery, CABEZA_ARR );
   }
   else if( serp->abajo )
   {
       DibujaBloque( display, pixmap, window, gc,
              serp->cabserx, serp->cabsery-1, SERPIENTE );
       DibujaBloque( display, pixmap, window, gc,
              serp->cabserx, serp->cabsery, CABEZA_ABJO );
   }
   else
   {
       DibujaBloque( display, pixmap, window, gc,
              serp->cabserx+1, serp->cabsery, SERPIENTE );
       DibujaBloque( display, pixmap, window, gc,
              serp->cabserx, serp->cabsery, CABEZA_IZQ );
   }
}


/*---------------------------------------------------------
  Draws the initial snake graphic (pointing down).
 --------------------------------------------------------*/
void DibujaSerpienteInicial( Display *display, Pixmap pixmap,
                       Window window, GC gc, struct serpiente *serp )
{
    DibujaBloque( display, pixmap, window, gc,
           serp->cabserx, serp->cabsery, CABEZA_ABJO );
}


/*---------------------------------------------------------
  Erases the tail if necessary.
 --------------------------------------------------------*/
void EliminaCola( Display *display, Pixmap pixmap,
                 Window window, GC gc, struct serpiente *serp )
{
 int direccion;

    direccion = cola[serp->colaserx][serp->colasery];
    DibujaBloque( display, pixmap, window, gc,
                 serp->colaserx, serp->colasery, VACIO );

    pantalla[serp->colaserx][serp->colasery] = VACIO;
    cola[serp->colaserx][serp->colasery] = VACIO;
    if( direccion == DIR_IZQ )
      serp->colaserx = (serp->colaserx)-1;
    if( direccion == DIR_DER )
      serp->colaserx = (serp->colaserx)+1;
    if( direccion == DIR_ARR )
      serp->colasery = (serp->colasery)-1;
    if( direccion == DIR_ABJO )
      serp->colasery = (serp->colasery)+1;
}


/*---------------------------------------------------------
  Erases the snake tail if there is not growing.
 --------------------------------------------------------*/
void BorraSerpiente( Display *display, Pixmap pixmap,
                      Window window, GC gc,
                      struct serpiente *serp )
{
  /* espera is a variable which indicates growing.
     If it's == 0 we can erase the tail, else we can
     not erase it (and the snake continues advancing)
     till becomes 0 */
  if ( serp->espera == 0 )
     EliminaCola( display, pixmap, window, gc, serp );
  else
      serp->espera = serp->espera - 1;
}


/*---------------------------------------------------------
  Draws the "TIPO" block we want to draw.
 --------------------------------------------------------*/
void DibujaBloque( Display *display, Drawable pixmap,
                   Drawable window, GC gc, int x, int y,
                   char tipo )
{

  XCopyArea(display, sprites[tipo], pixmap, gc,
        0, 0, anchura_bloque, altura_bloque,
        x*anchura_bloque, y*altura_bloque);
  XCopyArea(display, sprites[tipo], window, gc,
        0, 0, anchura_bloque, altura_bloque,
        x*anchura_bloque, y*altura_bloque);

}

/*--- Redraws the screen if necessary -------------------*/
void RedibujaPantalla( Display *display, Drawable drawable,
   GC gc, Pixmap pixmap, int x, int y, int anchura, int altura  )
{
   XCopyArea(display, pixmap, drawable, gc,
             x, y, anchura, altura, x, y);
}



/*--- Looks for colours from their names ----------------*/
unsigned long ColorPorNombre( Display *display, char *Nombre )
{
  XColor color, temp;
  XAllocNamedColor( display,
     DefaultColormap(display,DefaultScreen(display)),
     Nombre, &color, &temp );
  return( color.pixel );
}

/*--- Presentation screen (draw once when we die) -------*/
void Presentacion( Display *display, Drawable window,
                   Pixmap pixmap, GC gc )
{

  int wherey, i;
  char cad[10];

  /* the big XSnake title */
  XCopyArea(display, sprites[4], pixmap, gc, 0, 0, 238, 128,
            (anchura_bloque*(xmax))/2-(238/2), altura_bloque );

  /* draw all the text on the hi-score screen */
  XSetForeground(display, gc, blanco );
  wherey = ((altura_bloque*(ymax-1))/2) - ((NUM_RECORDS+2)/2);
  wherey -= 18;
  XDrawString( display, pixmap, gc,
       ((anchura_bloque*(xmax+1))/2)-((strlen(Records[idioma])*9)/2),
       wherey, Records[idioma], strlen(Records[idioma]) );
  wherey+=36;
  XSetForeground(display, gc, azul );
  XDrawString( display, pixmap, gc,
     ((anchura_bloque*(xmax+1))/2)-((strlen(NombreA[idioma])+10)*9),
     wherey,NombreA[idioma], strlen(NombreA[idioma]) );
  XDrawString( display, pixmap, gc,
     ((anchura_bloque*(xmax+1))/2)-((strlen(NivelA[idioma])*9)/2),
     wherey,NivelA[idioma], strlen(NivelA[idioma]) );
  XDrawString( display, pixmap, gc,
     ((anchura_bloque*(xmax+1))/2)+(9*9),
     wherey, PuntosA[idioma], strlen(PuntosA[idioma]) );
  wherey+=6;

  /* draw the hi-scores */
  XSetForeground(display, gc, amarillo );
  for(i = 0; i< NUM_RECORDS; i++ )
  {
    wherey += 18;
    XDrawString( display, pixmap, gc,
       ((anchura_bloque*(xmax+1))/2)-((strlen(records[i].nombre)+9)*9),
       wherey,records[i].nombre, strlen(records[i].nombre) );

    sprintf(cad, "%d", records[i].nivel );
    XDrawString( display, pixmap, gc,
       ((anchura_bloque*(xmax+1))/2)-((strlen(cad)*9)/2),
       wherey,cad, strlen(cad) );

    sprintf(cad, "%d", records[i].puntos );
    XDrawString( display, pixmap, gc,
       ((anchura_bloque*(xmax+1))/2)+(10*9),
       wherey,cad, strlen(cad) );
  }

  /* draw the help */
  XSetForeground(display, gc, azul );
  XDrawString( display, pixmap, gc,
       ((anchura_bloque*(xmax+1))/2)-((strlen(PulseF1[idioma])*9)/2),
       (altura_bloque*(ymax-1)),
       PulseF1[idioma], strlen(PulseF1[idioma]) );
  XDrawString( display, pixmap, gc,
       ((anchura_bloque*(xmax+1))/2)-((strlen(Instrucciones[idioma])*9)/2),
       (altura_bloque*(ymax-3)),
       Instrucciones[idioma], strlen(Instrucciones[idioma]) );

  /* we've written all to PIXMAP, flip it to screen */
  RedibujaPantalla(display, window, gc, pixmap,
  0, 0, anchura_bloque*(xmax+1), altura_bloque*(ymax+1)+40 );

}


/*--- Draw the Hi-score screen ----------------------------*/
void DibujaRecord( Display *display, Drawable pixmap,
           Pixmap window, GC gc, struct serpiente *serp )
{

  int wherey, i, mitad;
  char cad[10];

  XSetForeground(display, gc, negro );
  XFillRectangle( display, pixmap, gc, anchura_bloque,
     altura_bloque, anchura_bloque*(xmax-1), altura_bloque*(ymax-1));

  mitad = ((anchura_bloque*(xmax+1))/2);
  XCopyArea(display, sprites[4], pixmap, gc,
            0, 0, 238, 128,
            (anchura_bloque*(xmax))/2-(238/2), altura_bloque );

  XSetForeground(display, gc, blanco );
  wherey = ((altura_bloque*(ymax-1))/2) - ((NUM_RECORDS+2)/2);
  wherey -= 18;

  XDrawString( display, pixmap, gc,
       mitad-((strlen(Records2[idioma])*9)/2),
       wherey, Records2[idioma], strlen(Records2[idioma]) );

  wherey+=36;
  XSetForeground(display, gc, azul );
  XDrawString( display, pixmap, gc,
     mitad-((strlen(NombreA[idioma])+10)*9),
     wherey,NivelA[idioma], strlen(NivelA[idioma]) );

  XDrawString( display, pixmap, gc, mitad+(9*9),
     wherey, PuntosA[idioma], strlen(PuntosA[idioma]) );

  wherey+=20;
  XSetForeground(display, gc, amarillo );
  sprintf(cad, "%d", records[NUM_RECORDS].nivel );
  XDrawString( display, pixmap, gc,
     mitad-(14*9)-((strlen(cad)*9)/2),
     wherey,cad, strlen(cad) );

  sprintf(cad, "%d", serp->puntos );
  XDrawString( display, pixmap, gc, mitad+(10*9),
      wherey,cad, strlen(cad) );

  XDrawRectangle( display, pixmap, gc,
       (((anchura_bloque*(xmax+1))/2)-(6*9))-2,
       ((altura_bloque*(ymax-1))/2) - ((NUM_RECORDS+2)/2) + 60 -2,
       (10*9)+4,
       16+2);

  XSetForeground(display, gc, azul );
  XDrawString( display, pixmap, gc,
       mitad-((strlen(Records3[idioma])*9)/2),
       (altura_bloque*(ymax-1)),
       Records3[idioma], strlen(Records3[idioma]) );

  /* we've written all to PIXMAP, flip it to screen */
  RedibujaPantalla(display, window, gc, pixmap,
  0, 0, anchura_bloque*(xmax+1), altura_bloque*(ymax+1)+40 );

}


/*---------------------------------------------------------
  On this BIG function we get the user keystrokes to
  write his/her name on the score table. It works by
  using the tecla_ascii variable which is set to the
  latest key pressed on the main event loop.
 --------------------------------------------------------*/
void HazRecord( Display *display, Pixmap pixmap, Window window, GC gc,
                struct serpiente *serp )
{
  char cad[3];

  /* draws the initial cursor if offs = 0.
     offs is the global var which indicates where
     on the name string we're now */
  if( offs == 0 )
  {
       XSetForeground(display, gc, azul );
       cad[0] = '_';
       XDrawString( display, pixmap, gc,
            (((anchura_bloque*(xmax+1))/2)-(6*9))+((offs)*10),
           ((altura_bloque*(ymax-1))/2) - ((NUM_RECORDS+2)/2) + 72,
           cad, 1 );
        XDrawString( display, window, gc,
            (((anchura_bloque*(xmax+1))/2)-(6*9))+((offs)*10),
            ((altura_bloque*(ymax-1))/2) - ((NUM_RECORDS+2)/2) + 72,
            cad, 1 );
  }

  /* if there is no key available, return */
  if( tecla_ascii == 0 ) return;

  /* if we detect a key a-z, A-Z or 0-9, do this: */
  else if( (tecla_ascii>='a' && tecla_ascii<='z') ||
      (tecla_ascii>='A' && tecla_ascii<='Z') ||
      (tecla_ascii>='0' && tecla_ascii<='9') )
  {
     /* convert it to upper case */
     if(tecla_ascii>='a' && tecla_ascii<='z')
        tecla_ascii -= ('a'-'A');

     /* write it on the global nombre string */
     nombre[offs] = tecla_ascii;
     nombre[offs+1] = '\0';

     /* delete old cursor */
     if( offs < 8 )
     {
       XSetForeground(display, gc, negro );
       cad[0] = '_';
       XDrawString( display, pixmap, gc,
            (((anchura_bloque*(xmax+1))/2)-(6*9))+((offs)*10),
           ((altura_bloque*(ymax-1))/2) - ((NUM_RECORDS+2)/2) + 72,
           cad, 1 );
        XDrawString( display, window, gc,
            (((anchura_bloque*(xmax+1))/2)-(6*9))+((offs)*10),
            ((altura_bloque*(ymax-1))/2) - ((NUM_RECORDS+2)/2) + 72,
            cad, 1 );
      }

     cad[0] = tecla_ascii;
     tecla_ascii = 0;

     /* draw the key pressed */
     XSetForeground(display, gc, azul );
     XDrawString( display, pixmap, gc,
          (((anchura_bloque*(xmax+1))/2)-(6*9))+(offs*10),
          ((altura_bloque*(ymax-1))/2) - ((NUM_RECORDS+2)/2) + 72,
          cad, 1 );
     XDrawString( display, window, gc,
          (((anchura_bloque*(xmax+1))/2)-(6*9))+(offs*10),
          ((altura_bloque*(ymax-1))/2) - ((NUM_RECORDS+2)/2) + 72,
          cad, 1 );

     if( offs < 8 )
     {
       /* draw the new cursor */
       cad[0] = '_';
       XDrawString( display, pixmap, gc,
            (((anchura_bloque*(xmax+1))/2)-(6*9))+((offs+1)*10),
           ((altura_bloque*(ymax-1))/2) - ((NUM_RECORDS+2)/2) + 72,
           cad, 1 );
        XDrawString( display, window, gc,
            (((anchura_bloque*(xmax+1))/2)-(6*9))+((offs+1)*10),
            ((altura_bloque*(ymax-1))/2) - ((NUM_RECORDS+2)/2) + 72,
            cad, 1 );
      }

     /* advance the offset. If we've reached the end
        of the text cell, write the record */
     if( offs < 8 ) offs++;
     else tecla_ascii = ENTER;
  }

  /* if the user press BACKSPACE, erase the last key drawn */
  else if( tecla_ascii == '\b' && offs > 0 )
  {

     XSetForeground(display, gc, negro );
     cad[0] = '_';
     XDrawString( display, pixmap, gc,
          (((anchura_bloque*(xmax+1))/2)-(6*9))+((offs)*10),
         ((altura_bloque*(ymax-1))/2) - ((NUM_RECORDS+2)/2) + 72,
         cad, 1 );
      XDrawString( display, window, gc,
          (((anchura_bloque*(xmax+1))/2)-(6*9))+((offs)*10),
          ((altura_bloque*(ymax-1))/2) - ((NUM_RECORDS+2)/2) + 72,
          cad, 1 );

     offs--;
     cad[0] = nombre[offs];
     nombre[offs]='\0';
     XSetForeground(display, gc, negro );
     XDrawString( display, pixmap, gc,
          (((anchura_bloque*(xmax+1))/2)-(6*9))+(offs*10),
          ((altura_bloque*(ymax-1))/2) - ((NUM_RECORDS+2)/2) + 72,
          cad, 1 );
     XDrawString( display, window, gc,
          (((anchura_bloque*(xmax+1))/2)-(6*9))+(offs*10),
          ((altura_bloque*(ymax-1))/2) - ((NUM_RECORDS+2)/2) + 72,
          cad, 1 );

      XSetForeground(display, gc, azul );
      cad[0] = '_';
      XDrawString( display, pixmap, gc,
           (((anchura_bloque*(xmax+1))/2)-(6*9))+((offs)*10),
           ((altura_bloque*(ymax-1))/2) - ((NUM_RECORDS+2)/2) + 72,
           cad, 1 );
       XDrawString( display, window, gc,
           (((anchura_bloque*(xmax+1))/2)-(6*9))+((offs)*10),
           ((altura_bloque*(ymax-1))/2) - ((NUM_RECORDS+2)/2) + 72,
           cad, 1 );

     tecla_ascii = 0;
  }

  /* if the user press ENTER and offs > 0 (he has written
     something) then we've done and put the hi-score! */
  if( tecla_ascii == ENTER && offs > 0)
  {
      nombre[offs+1]='\0';
      strcpy(records[NUM_RECORDS].nombre, nombre );
      records[NUM_RECORDS].puntos = serp->puntos;
      OrdenaRecords();
      EscribeRecords();
      records[NUM_RECORDS].puntos = 0;

      hay_record = 0;  /* this makes the game start another
                          time on the presentation screen */
      offs = 0;
  }
  tecla_ascii = 0;
}

/*=== HI-SCORE functions ================================*/
/*---------------------------------------------------------
  This function tries to read the .xsnake.hi file on the
  HOME directory of the user. If it does not exist, it
  creates a new hi-score set (with a loop).
 --------------------------------------------------------*/
void LeeRecords( void )
{

  FILE *fp;
  char *home;
  char fichero[256];
  int i;

  home = getenv("HOME");
  sprintf(fichero, "%s/.xsnake.hi", home );
  fp = fopen( fichero, "rt" );

  /* if the file does not exist... */
  if( fp == NULL )
  {
    /* create new stupid hi-scores */
    for( i=0; i<NUM_RECORDS; i++ )
    {
      strcpy(records[i].nombre,"SROMERO");
      records[i].puntos = (4-i+1)*100;
      records[i].nivel = (4-i+1);
    }
    EscribeRecords();
    return;
  }

  /* else read them! */
  for( i=0; i<NUM_RECORDS; i++ )
  {
     fscanf(fp, "%s", records[i].nombre );
     fscanf(fp, "%d", &(records[i].puntos) );
     fscanf(fp, "%d", &(records[i].nivel) );
  }
  fclose(fp);
}


/*---------------------------------------------------------
  Writes the hi-scores of records[0 to 4] on xsnake.hi...
 --------------------------------------------------------*/
void EscribeRecords( void )
{
  FILE *fp;
  char *home;
  char fichero[256];
  int i;

  home = getenv("HOME");
  sprintf(fichero, "%s/.xsnake.hi", home );
  fp = fopen( fichero, "wt" );

  for( i=0; i<NUM_RECORDS; i++ )
  {
     fprintf(fp, "%s\n", records[i].nombre );
     fprintf(fp, "%d\n", records[i].puntos );
     fprintf(fp, "%d\n", records[i].nivel );
  }
  fclose(fp);
}


/*---------------------------------------------------------
  This sort function is used to insert the player score
  on the hi-score table. We have 5 hi-scores on
  Records[0] to Records[4], and we put the player's
  score on Records[5]. Then we call SortScores()
  (OrdenarRecords()) and we get the player's score
  on the right position :)
 --------------------------------------------------------*/
void OrdenaRecords( void )
{
  int i, j, numelem;
  struct record temp;

  numelem = NUM_RECORDS+1;

  for( i=1; i<numelem; i++ )
  {
      for( j=1; j<numelem; j++ )
      {
          if( records[j].puntos > records[j-1].puntos )
          {
              temp = records[j];
              records[j] = records[j-1];
              records[j-1] = temp;
          }
      }
  }
}
